/*
 * PlayerScreen.java
 *
 * Copyright (c) David Pollack 2009
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package ScrobbleBerry;

import java.io.*;
import java.util.Timer;
import java.util.TimerTask;
import javax.microedition.media.*;
import javax.microedition.media.control.*;
import net.rim.device.api.ui.Keypad;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.*;
import net.rim.device.api.system.*;


final class PlayerScreen extends MainScreen implements PlayerListener {
    String _artist = "";
    String _track = "";
    String _album = "";
    String _tracknumber = "";
    String _length = "";
    long _timestamp;
    
    ScrobblerSession _session = null;
    
    RichTextField _trackField =
            new RichTextField(RichTextField.TEXT_ALIGN_HCENTER);
    RichTextField _artistField =
            new RichTextField(RichTextField.TEXT_ALIGN_HCENTER);
    RichTextField _albumField =
            new RichTextField(RichTextField.TEXT_ALIGN_HCENTER);
    RichTextField _durationField =
            new RichTextField(RichTextField.TEXT_ALIGN_HCENTER |
                                RichTextField.NON_FOCUSABLE);
    
    Player p = null;
    String[] playList = null;
    int playIndex = 0;
    
    ControlKeyListener listener = new ControlKeyListener();
    
    PlayerScreen(String[] uris, ScrobblerSession session) {
        _session = session;
        playList = uris;
        playIndex = 0;
        
        LabelField _title = new LabelField("ScrobbleBerry - Now Playing",
                    LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
        setTitle(_title);
        
        add(_trackField);
        add(new SeparatorField());
        add(_artistField);
        add(new SeparatorField());
        add(_albumField);
        add(new SeparatorField());
        add(_durationField);
        
        playNew(playList[playIndex]);
    }
    
    void playNew(String uri) {
        try {
            p = Manager.createPlayer(uri);
            p.addPlayerListener(this);
            p.realize();
            p.prefetch();
            
            try {
                long d;
                if ((d = p.getDuration()) != Player.TIME_UNKNOWN) {
                    _length = Long.toString(d / 1000000);
                }
            } catch (IllegalStateException e) {
            }
            
            MetaDataControl m = (MetaDataControl)
                        p.getControl("MetaDataControl");
            
            try {
                _artist = m.getKeyValue(MetaDataControl.AUTHOR_KEY);
                _track = m.getKeyValue(MetaDataControl.TITLE_KEY);
            } catch (IllegalArgumentException e) {
                // mandatory tags
                _artist = null;
                _track = null;
            }
            
            try {
                _album = m.getKeyValue("album");
                _tracknumber = m.getKeyValue("track number");
            } catch (IllegalArgumentException e) {
                // optional tags
            }
            
            if (_artist == null) _artist = "";
            if (_track == null) _track = "";
            if (_album == null) _album = "";
            if (_tracknumber == null) _tracknumber = "";
            
            _trackField.setText(_track);
            _artistField.setText(_artist);
            _albumField.setText(_album);
            _durationField.setText("0:00 / " + convertMins(_length));
            
            if (!_artist.equals("") && !_track.equals("")) {
                _timestamp = System.currentTimeMillis() / 1000;
                _session.sendNowPlaying(_artist, _track, _album,
                    _length, _tracknumber);
            }
            
            p.start();
            
            if (playIndex == 0) {
                TimerTask _updateDuration = new TimerTask() {
                    public void run() {
                        if (p != null) {
                            long currTime;
                            try {
                                currTime = p.getMediaTime() / 1000000;
                            } catch (IllegalStateException e) {
                                currTime = Player.TIME_UNKNOWN;
                            }
                            if (currTime != Player.TIME_UNKNOWN) {
                                _durationField.setText(convertMins(currTime) +
                                        " / " + convertMins(_length));
                            }
                        }
                    }
                };
                
                Timer t = new Timer();
                t.schedule(_updateDuration, 0, 500);
            }
        } catch (MediaException e) {
            Status.show(uri + " could not be played.", 1000);
            advance();
        } catch (IOException e) {
            Status.show(uri + " could not be opened.", 1000);
            advance();
        }
    }
    
    private String convertMins(long secs) {
        return (secs / 60) + ":" +
            (((secs % 60) < 10)?("0" + (secs % 60)):Long.toString(secs % 60));
    }
    
    private String convertMins(String secs) {
        return convertMins(Long.parseLong(secs));
    }
    
    public void close() {
        handleStop();
        super.close();
    }
    
    void advance() {
        if (p != null) {
            p.close();
        }
        
        playIndex++;
        if (playIndex < playList.length) {
            playNew(playList[playIndex]);
        } else {
            super.close();
        }
    }
    
    void handleStop() {
        if (p != null) {
            long stopTime;
            try {
                p.stop();
                stopTime = p.getMediaTime();
            } catch (IllegalStateException e) {
                stopTime = Player.TIME_UNKNOWN;
            } catch (MediaException e) {
                stopTime = Player.TIME_UNKNOWN;
            }
            if (stopTime != Player.TIME_UNKNOWN) {
                handleScrobble(stopTime, p.getDuration());
            }
            p.close();
        }
    }
    
    void handleScrobble(long stopTime, long totalTime) {
        double stopTimeSecs = stopTime / 1000000;
        double totalTimeSecs = totalTime / 1000000;
        
        if (totalTimeSecs > 30) {
            double scrobbleTimeSecs;
            
            if (totalTimeSecs > 480) {
                scrobbleTimeSecs = 240;
            } else {
                scrobbleTimeSecs = totalTimeSecs / 2;
            }
            
            if (stopTimeSecs >= scrobbleTimeSecs &&
                !_artist.equals("") && !_track.equals("")) {
                _session.sendScrobble(_artist, _track, _album,
                                _length, _tracknumber, _timestamp);
            }
        }
    }
    
    public void playerUpdate(final Player player,
                                final String event,
                                final Object eventData) {
        UiApplication.getUiApplication().invokeLater(new Runnable()
        {
            public void run()
            {
                if (event.equals(END_OF_MEDIA)) {
                    handleStop();
                    advance();
                }
            }
        });
    }
    
    private class ControlKeyListener
            extends MainScreen implements KeyListener {
        public ControlKeyListener() {
            Application.getApplication().addKeyListener(this);
        }
        
        public boolean keyChar(char key, int status, int time) {
            return false;
        }
        public boolean keyDown(int keycode, int time) {
            if (Keypad.key(keycode) == Keypad.KEY_VOLUME_DOWN) {
                if (p != null && p.getState() != Player.REALIZED) {
                    VolumeControl v = (VolumeControl)
                                p.getControl("VolumeControl");
                    int currLevel = v.getLevel();
                    if (currLevel < 10) {
                        currLevel = 0;
                    } else {
                        currLevel -= 10;
                    }
                    currLevel = v.setLevel(currLevel);
                }
                return true;
            } else if (Keypad.key(keycode) == Keypad.KEY_VOLUME_UP) {
                if (p != null && p.getState() != Player.REALIZED) {
                    VolumeControl v = (VolumeControl)
                                p.getControl("VolumeControl");
                    int currLevel = v.getLevel();
                    if (currLevel > 90) {
                        currLevel = 100;
                    } else {
                        currLevel += 10;
                    }
                    currLevel = v.setLevel(currLevel);
                }
                return true;
            } else if (Keypad.key(keycode) == Keypad.KEY_SPEAKERPHONE) {
                if (p != null) {
                    try {
                        if (p.getState() == Player.STARTED) {
                            p.stop();
                        } else {
                            p.start();
                        }
                    } catch (MediaException e) {
                        Dialog.alert("Media could not be started/stopped.");
                    }
                }
                return true;
            }
            return false;
        }
        public boolean keyRepeat(int keycode, int time) {
            return false;
        }
        public boolean keyStatus(int keycode, int time) {
            return false;
        }
        public boolean keyUp(int keycode, int time) {
            return false;
        }
    }
}

